<?php
// +--------------------------------------------------------------------------+
// | PayPal Plugin - geeklog CMS                                             |
// +--------------------------------------------------------------------------+
// | ipnlog.php                                                               |
// |                                                                          |
// | Allows paypal administrators to view the ipnlog from the database        |
// +--------------------------------------------------------------------------+
// |                                                                          |
// | Copyright (C) 2005-2006 by the following authors:                        |
// |                                                                          |
// | Authors: Vincent Furia     - vinny01 AT users DOT sourceforge DOT net    |
// +--------------------------------------------------------------------------+
// |                                                                          |
// | This program is free software; you can redistribute it and/or            |
// | modify it under the terms of the GNU General Public License              |
// | as published by the Free Software Foundation; either version 2           |
// | of the License, or (at your option) any later version.                   |
// |                                                                          |
// | This program is distributed in the hope that it will be useful,          |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of           |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            |
// | GNU General Public License for more details.                             |
// |                                                                          |
// | You should have received a copy of the GNU General Public License        |
// | along with this program; if not, write to the Free Software Foundation,  |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.          |
// |                                                                          |
// +--------------------------------------------------------------------------+

/**
 * Allows paypal administrators to view the ipnlog from the database
 *
 * @author Vincent Furia <vinny01 AT users DOT sourceforge DOT net>
 * @copyright Vincent Furia 2005 - 2006
 * @package paypal
 */

/**
 * Required geeklog
 */
require_once('../../../lib-common.php');

// Check for required permissions
paypal_access_check('paypal.admin');

// Incoming variable filter
$vars = array('op' => 'alpha',
              'txn_id' => 'alpha',
              'id' => 'number');
paypal_filterVars($vars, $_REQUEST);

/**
 * Displays the list of ipn history from the log stored in the database
 *
 */
function PAYPAL_listIPNlog()
{
    global $_CONF, $_TABLES, $LANG_PAYPAL_1, $_USER;

    require_once $_CONF['path_system'] . 'lib-admin.php';

    $retval = '';

	if (DB_count($_TABLES['paypal_ipnlog']) == 0){
	    $retval .= '<p>' . $LANG_PAYPAL_1['ipnlog_empty'] . '</p>';
	}


    $header_arr = array(      // display 'text' and use table field 'field'
        array('text' => $LANG_PAYPAL_1['ID'], 'field' => 'id', 'sort' => true),
        array('text' => $LANG_PAYPAL_1['IP_address'], 'field' => 'ip_addr', 'sort' => true),
        array('text' => $LANG_PAYPAL_1['date_time'], 'field' => 'time', 'sort' => true),
        array('text' => $LANG_PAYPAL_1['verified'], 'field' => 'verified', 'sort' => true),
		array('text' => $LANG_PAYPAL_1['txn_id'], 'field' => 'txn_id', 'sort' => true),
		array('text' => $LANG_PAYPAL_1['payment_status'], 'field' => 'payment_status', 'sort' => true),
        array('text' => $LANG_PAYPAL_1['purchaser'], 'field' => 'custom', 'sort' => true)
    );
    $defsort_arr = array('field' => 'id', 'direction' => 'desc');

    $text_arr = array(
        'has_extras' => true,
        'form_url' => $_CONF['site_admin_url'] . '/plugins/paypal/ipnlog.php'
    );
	
	$sql = "SELECT * FROM {$_TABLES['paypal_ipnlog']} WHERE 1=1";

    $query_arr = array(
        'table'          => 'paypal_ipnlog',
        'sql'            => $sql,
        'query_fields'   => array('id', 'ip_addr', 'time', 'verified', 'txn_id', 'ipn_data'),
        'default_filter' => COM_getPermSQL ('AND', 0, 3)
    );

    $retval .= ADMIN_list('paypal', 'plugin_getListField_paypal_IPNlog',
                          $header_arr, $text_arr, $query_arr, $defsort_arr);

    return $retval;
}

/**
*   Get an individual field for the paypal screen.
*
*   @param  string  $fieldname  Name of field (from the array, not the db)
*   @param  mixed   $fieldvalue Value of the field
*   @param  array   $A          Array of all fields from the database
*   @param  array   $icon_arr   System icon array
*   @param  object  $EntryList  This entry list object
*   @return string              HTML for field display in the table
*/
function plugin_getListField_paypal_IPNlog($fieldname, $fieldvalue, $A, $icon_arr)
{
    global $_CONF, $_PAY_CONF, $LANG_PAYPAL_1;
	
	$out = preg_replace('!s:(\d+):"(.*?)";!se', "'s:'.strlen('$2').':\"$2\";'", $A['ipn_data'] ); 
    $ipn = unserialize($out);
	if (!is_array($ipn)) {
        $ipn = array();
    }

    switch($fieldname) {
        case "id":
            $retval = $A['id'];
            break;
		case "ip_addr":
            $retval = $A['ip_addr'];
            break;
		case "time":
            $retval = $A['time'];
            break;
		case "verified":
            $retval = $A['verified'];
            break;
		case "txn_id":
            $retval = '<a href="' . $_CONF['site_url'] . '/admin/plugins/paypal/ipnlog.php?view=ipnlog&op=single&txn_id=' . $A['txn_id'] . '">' . $A['txn_id'] . '</a>';
            break;
		case "payment_status":
            $retval = $ipn['payment_status'];
            break;
		case "custom":
            ($ipn['custom'] > 1) ? $retval = '<a href="' . $_CONF['site_url'] . '/users.php?mode=profile&uid=' . $ipn['custom'] . '">' . $ipn['last_name'] . '</a>' . ' (' . $ipn['custom'] . ')' : $retval = $ipn['last_name'] ;
            break;

        default:
            $retval = stripslashes($fieldvalue);
            break;
    }
    return $retval;
}

/**
 * Display a single row from the ipnlog
 *
 * Not only display the row, but beautify the serialized POST data from the transaction
 * for easy viewing.
 *
 * @param int $id Unique identifier for the row of the ipnlog to display
 * @param string $txn_id Unique identifier for the row (txn_id) of the ipnlog to display
 * @return string HTML of the ipnlog row specified by $id
 */
function PAYPAL_ipnlog_single($id, $txn_id) {
    global $_TABLES, $LANG_PAYPAL_1, $_CONF;

    // Get ipnlog from database
    if ($id > 0) {
        $sql = "SELECT * FROM {$_TABLES['paypal_ipnlog']} WHERE id = $id";
    } else {
        $sql = "SELECT * FROM {$_TABLES['paypal_ipnlog']} WHERE txn_id = '$txn_id'";
    }
    $res = DB_query($sql);
    $A = DB_fetchArray($res);

    // Start Display
    $display .= COM_startBlock($LANG_PAYPAL_1['ipn_history'] . " (#{$A['id']})");

    // Create ipnlog template
    $ipnlog = new Template($_CONF['path'] . 'plugins/paypal/templates');
    $ipnlog->set_file(array('ipnlog' => 'ipnlog_detail.thtml'));
    $ipnlog->set_var('site_url', $_CONF['site_url']);
    $ipnlog->set_var('IPN_log', $LANG_PAYPAL_1['IPN_log']);
    $ipnlog->set_var('IP_address', $LANG_PAYPAL_1['IP_address']);
    $ipnlog->set_var('date_time', $LANG_PAYPAL_1['date_time']);
    $ipnlog->set_var('Verified', $LANG_PAYPAL_1['verified']);
    $ipnlog->set_var('transaction', $LANG_PAYPAL_1['transaction']);
    $ipnlog->set_var('gross_payment', $LANG_PAYPAL_1['gross_payment']);
    $ipnlog->set_var('payment_status', $LANG_PAYPAL_1['payment_status']);
	$ipnlog->set_var('ipn_data', $LANG_PAYPAL_1['ipn_data']);

    // Display the specified ipnlog row
    $ipnlog->set_var('id', $A['id']);
    $ipnlog->set_var('ip_addr', $A['ip_addr']);
    $ipnlog->set_var('time', $A['time']);
    if ($A['verified']) {
        $ipnlog->set_var('verified', $LANG_PAYPAL_1['true']);
    } else {
        $ipnlog->set_var('verified', $LANG_PAYPAL_1['false']);
    }

    // Allow all serialized data to be available to the template
	$ipn ='';
	if ($A['ipn_data'] != '') {
	    $out = preg_replace('!s:(\d+):"(.*?)";!se', "'s:'.strlen('$2').':\"$2\";'", $A['ipn_data'] ); 
        $ipn = unserialize($out);
		if (!is_array($ipn)) {
            $ipn = array();
        }
        foreach ($ipn as $name => $value) {
            $ipnlog->set_var($name, $value);
        }
	}

    // Grab a raw print of the ipn data
    ob_start();
    print_r($ipn);
    $raw = ob_get_contents();
    ob_end_clean();

    // replace \n with <br>
    $raw = nl2br($raw);

    $ipnlog->set_var('raw', $raw);
    $display .= $ipnlog->parse('output', 'ipnlog');

    $display .= COM_endBlock();
    return $display;

}

//Main

$display = COM_siteHeader('none');
$display .= paypal_admin_menu();

// base output on selected opeation (op)
switch ($_REQUEST['op']) {
    case 'single':
        $display .= PAYPAL_ipnlog_single($_REQUEST['id'], $_REQUEST['txn_id']);
        break;

    case 'all':
    default:
        $display .= COM_startBlock($LANG_PAYPAL_1['IPN_logs']);
        $display .= PAYPAL_listIPNlog();
        $display .= COM_endBlock();
        break;
}

$display .= COM_siteFooter();
echo $display;

?>