<?php
// +--------------------------------------------------------------------------+
// | PayPal Plugin - geeklog CMS                                              |
// +--------------------------------------------------------------------------+
// | BaseIPN.class.php                                                        |
// |                                                                          |
// | This file contains the BaseIPN class, it provides an interface to        |
// | deal with IPN transactions from paypal                                   |
// +--------------------------------------------------------------------------+
// |                                                                          |
// | Copyright (C) 2011 by the following authors:                             |
// |                                                                          |
// | Authors: Ben     -    ben AT geeklog DOT fr                              |
// +--------------------------------------------------------------------------+
// |                                                                          |
// | Copyright (C) 2005-2006 by the following authors:                        |
// |                                                                          |
// | Authors: Vincent Furia     - vinny01 AT users DOT sourceforge DOT net    |
// +--------------------------------------------------------------------------+
// |                                                                          |
// | This program is free software; you can redistribute it and/or            |
// | modify it under the terms of the GNU General Public License              |
// | as published by the Free Software Foundation; either version 2           |
// | of the License, or (at your option) any later version.                   |
// |                                                                          |
// | This program is distributed in the hope that it will be useful,          |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of           |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            |
// | GNU General Public License for more details.                             |
// |                                                                          |
// | You should have received a copy of the GNU General Public License        |
// | along with this program; if not, write to the Free Software Foundation,  |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.          |
// |                                                                          |
// +--------------------------------------------------------------------------+

// this file can't be used on its own
if (!defined ('VERSION')) {
    die ('This file can not be used on its own.');
}

/**
 * This file contains the BaseIPN class, it provides an interface to deal with
 * IPN transactions from paypal
 *
 * @author Vincent Furia <vinny01 AT users DOT sourceforge DOT net>
 * @copyright Vincent Furia 2005 - 2006
 * @package paypal
 */
 
/* Paypal constants */
define('PAYPAL_FAILURE_UNKNOWN', 0);
define('PAYPAL_FAILURE_VERIFY', 1);
define('PAYPAL_FAILURE_COMPLETED', 2);
define('PAYPAL_FAILURE_UNIQUE', 3);
define('PAYPAL_FAILURE_EMAIL', 4);
define('PAYPAL_FAILURE_FUNDS', 5);

/**
 * This class provides an interface to deal with IPN transactions from paypal
 *
 * @package paypal
 */
class BaseIPN {

    /**
     * Verify the transaction by check Paypal's servers
     *
     * Validate transaction by posting data back to the paypal webserver.  The response from
     * paypal should include 'VERIFIED' on a line by itself.
     *
     * @param array $in Array containing POST variables of transaction
     * @return boolean true if result successfully validated, false otherwise
     */
    function Verify($in) 
	{
        global $_CONF, $_PAY_CONF;

        // Default verification to false
        $verified = false;

        // read the post from PayPal system and add 'cmd'
        $req = 'cmd=_notify-validate';

        // re-encode the transaction variables to be verified
        foreach ($in as $key => $value) {
            $value = urlencode(stripslashes($value));
            $req .= "&$key=$value";
        }

        // post back to PayPal system to validate
        $header .= "POST /cgi-bin/webscr HTTP/1.0\r\n";
        $header .= "Content-Type: application/x-www-form-urlencoded\r\n";
        $header .= "Content-Length: " . strlen($req) . "\r\n\r\n";
        $fp = fsockopen ($_PAY_CONF['paypalURL'], 80, $errno, $errstr, 30);

        if (!$fp) {
            $this->handleFailure(PAYPAL_FAILURE_UNKNOWN, 
			    "Could not connect to paypal URL for verification");
            return false;
        } else {
            fputs ($fp, $header . $req);
            while (!feof($fp)) {
                $res = fgets ($fp, 1024);
                // For Debug Only
                if ($_PAY_CONF['debug']) COM_errorLog("PAYPAL-IPN: IPN result -- $res");
                if (strcmp ($res, 'VERIFIED') == 0) {
                    $verified = true;
                    break;
                } else if (strcmp ($res, 'INVALID') == 0) {
                    $verified = false;
                    break;
                }
            }
            fclose ($fp);
        }
        return $verified;
    }

    /**
     * Log an IPN
     *
     * Logs the incoming IPN (serialized) along with the time it arrived, the originating IP address
     * and whether or not it has been verified (caller specified).  Also inserts the txn_id
     * seperately for look-up purposes.
     *
     * @param array $in POST variables of IPN transaction
     * @param boolean $verified true if transaction has been verified, false otherwise
     */
    function Log($in, $verified = false) {
        global $_SERVER, $_TABLES;

        // Change $verified into format for database
        if ($verified) {
            $verified = 1;
        } else {
            $verified = 0;
        }

        //Check if IPN already exists
		$id = DB_getItem($_TABLES['paypal_ipnlog'], 'id', "txn_id='{$in['txn_id']}'");
		if ( $id == '') {
			// Log to database
			$sql = "INSERT INTO {$_TABLES['paypal_ipnlog']} SET ip_addr = '{$_SERVER['REMOTE_ADDR']}', "
				 . "time = NOW(), verified = $verified, txn_id = '{$in['txn_id']}', "
				 . "ipn_data = '" . serialize($in) . '\'';
			DB_query($sql);
			if ($_PAY_CONF['debug']) COM_errorLog('IPN recorded');
			return DB_insertId();
		} else {
		    return $id;
		}
    }

    /**
     * Returns true if the the supplied email address is amoung allowed receiver addresses
     *
     * @param string $receiver_email Email to verify
     * @return boolean true if valid, false otherwise
     */
    function isValidEmail($receiver_email, $business) {
        global $_PAY_CONF;

        if ( ($receiver_email == $_PAY_CONF['receiverEmailAddr']) || $business == $_PAY_CONF['receiverEmailAddr'] ) {
		    if ($_PAY_CONF['debug']) COM_errorLog('Email ok');
		    return true;
		} else {
		    if ($_PAY_CONF['debug']) COM_errorLog('Email not ok');
		    return false;
		}
    }

    /**
     * Checks to make sure that the transaction id is unique to prevent double counting.
     *
     * @param string $txn_id transaction id to verify
     * @return boolean true if unique, false otherwise
     */
    function isUniqueTxnId($txn_id) {
        global $_TABLES;

        // Count purchases with txn_id, if > 0
        $count = DB_count($_TABLES['paypal_purchases'], 'txn_id', $txn_id);
        if ($count > 0) {
		    if ($_PAY_CONF['debug']) COM_errorLog('Txn is not unique');
            return false;
        } else {
		    if ($_PAY_CONF['debug']) COM_errorLog('Txn is unique');
            return true;
        }
    }

    /**
     * Confirms that payment status is complete (not 'denied', 'failed', 'pending', etc.)
     *
     * @param string $payment_status payment status to verify
     * @return boolean true if complete, false otherwise
     */
    function isStatusCompleted($payment_status) {
        return ($payment_status == 'Completed');
    }

    /**
     * Checks if payment status is reversed or refunded (ie some sort of cancelation)
     *
     * @param string $payment_status payment status to check
     * @return boolean true if payment status is reversed or refunded, false otherwise
     */
    function isStatusReversed($payment_status) {
        return ($payment_status == 'Reversed' || $payment_status == 'Refunded');
    }

    /**
     * Checks to make sure provided funds are sufficient to cover the cost of the purchased item(s)
     *
     * @param array $ids ids of items to check
     * @param array $quantity number ordered (per item)
     * @param real $payment_gross total payment made in current transaction
     * @param string $currency Currency of funds in payment_gross
     * @return boolean true if funds are sufficient, false otherwise
     */
    function isSufficientFunds($ids, $quantity, $payment_gross, $currency) {
        global $_CONF, $_PAY_CONF, $_TABLES;
		
		$ids = PAYPAL_realId($ids);

        // Check currency
        if (!(isset($_PAY_CONF['currency']) && strcasecmp($_PAY_CONF['currency'], $currency) == 0)) {
		    if ($_PAY_CONF['debug']) COM_errorLog('Currency is not ok');
            return false;
        }

        // Create a list of ids from $ids
        $idlist = implode(',', $ids);

        // Create/execute query string
        $sql = "SELECT * FROM {$_TABLES['paypal_products']} WHERE id in ($idlist)";
        $res = DB_query($sql);

        // Create a price lookup table
		//TODO add attribute price
        while ($A = DB_fetchArray($res)) {
		    $price = $A['price'];
			if ($A['discount_a'] != '' && $A['discount_a'] != 0) {
				$price = number_format($A['price'] - $A['discount_a'], 2, '.', '');
			}
			if ($A['discount_p'] != '' && $A['discount_p'] != 0) {
				$price = number_format($A['price'] - ($A['price'] * ($A['discount_p']/100)), 2, '.', '');
			}
            $cost[$A['id']] = $price;
        }

        // calculate the total purchase price
        $total = 0;
        for ($i = 0; $i < count($ids); $i++) {
            $total += $cost[$ids[$i]] * $quantity[$i];
        }

        // Compare total price to gross payment
        if ($total <= $payment_gross) {
		    if ($_PAY_CONF['debug']) COM_errorLog('Funds are sufficient');
            return true;
        } else {
		    if ($_PAY_CONF['debug']) COM_errorLog('Funds are not sufficient');
            //Todo send a mail to admin
            return false;
        }
    }

    /**
     * Process an incoming IPN transaction
     *
     * Do the following:
     * <ol>
     * <li>verify IPN</li>
     * <li>Log IPN</li>
     * <li>Check that transaction is complete</li>
     * <li>Check that transaction is unique</li>
     * <li>Check for valid receiver email address</li>
     * <li>Process IPN</li>
     * </ol>
     *
     * @param array $in POST variables of transaction
     * @return boolean true if processing valid and completed, false otherwise
     */
    function Process($in) {
	
	    global $_PAY_CONF;
		
	    if ($_PAY_CONF['debug']) COM_errorLog('IPN received');
		
        if (!$this->Verify($in)) {
            $logId = $this->Log($in, false);
            $this->handleFailure(PAYPAL_FAILURE_VERIFY, "($logId) Verification failed");
            return false;
        } else {
            $logId = $this->Log($in, true);
        }

        if (!$this->isStatusCompleted($in['payment_status'])) {
            // Not logged since this probably isn't an error
            // $this->handleFailure(PAYPAL_FAILURE_COMPLETED, "($logId) Status not complete");
            return false;
        }

        if (!$this->isUniqueTxnId($in['txn_id'])) {
            $this->handleFailure(PAYPAL_FAILURE_UNIQUE, "($logId) Non-unique transaction id");
            return false;
        }

        if (!$this->isValidEmail($in['receiver_email'], $in['business'])) {
            $this->handleFailure(PAYPAL_FAILURE_EMAIL, "($logId) Invalid receiver email address");
            return false;
        }

        switch ($in['txn_type']) {
            // buy now, donate, smart logos
            case 'web_accept':  //usually buy now
            case 'send_money':  //usually donation/send money
                // Process Buy Now, ignore donations
                if (!empty($in['item_number'])) {
                    $ids = array($in['item_number']);
                    $quantity = array($in['quantity']);
					$name = array($in['item_name']);
                    if (isset($in['settle_amount'])) {
                        $payment_gross = $in['mc_gross'] * $in['exchange_rate'];
                        $currency      = $in['settle_currency'];
                    } else {
                        $payment_gross = $in['mc_gross'];
                        $currency      = $in['mc_currency'];
                    }
                    if ($this->isSufficientFunds($ids, $quantity, $payment_gross, $currency)) {
                        $this->handlePurchase($ids, $quantity, $in, $name);
                    } else {
                        $this->handleFailure(PAYPAL_FAILURE_FUNDS, "($logId) Insufficient funds for purchase");
                        return false;
                    }
                } else {
                    $this->handleDonation($in);
                }
                break;

            // shopping cart
            case 'cart':
                $ids = array();
                $quantity = array();
                for ($i = 1; $i <= $in['num_cart_items']; $i++) {
					$ids[] = $in["item_number$i"];
                    $quantity[] = $in["quantity$i"];
					$names[] = $in["item_name$i"];
                }
                if (isset($in['settle_amount'])) {
                    $payment_gross = $in['mc_gross'] * $in['exchange_rate'];
                    $currency      = $in['settle_currency'];
                } else {
                    $payment_gross = $in['mc_gross'];
                    $currency      = $in['mc_currency'];
                }
                if ($this->isSufficientFunds($ids, $quantity, $payment_gross, $currency)) {
                    $this->handlePurchase($ids, $quantity, $in, $names);
                } else {
                    $this->handleFailure(PAYPAL_FAILURE_FUNDS, "($logId) Insufficient/incorrect funds for purchase");
                    return false;
                }
                break;

            // other, unknown, unsupported
            default:
                $this->handleFailure(PAYPAL_FAILURE_UNKNOWN, "($logId) Unknown transaction type");
                return false;
        }

        COM_errorLog("PAYPAL-IPN: purchases success",1);
        return true;
    }

    /**
     * Add a record of the purchase to the DB
     *
     * @param array $products Product Id(s) of Product(s) purchased
     * @param array $quantity Quantity of products purchases
     * @param array $paypal_data IPN POST variables
     * @todo implemente physical item vs. download, reflected in 'status'
     */
    function handlePurchase($products, $quantity, $paypal_data, $product_name) {
        global $_TABLES, $_CONF, $_PAY_CONF, $LANG_PAYPAL_EMAIL;

        // initialize file and names arrays
        $files = array();
        $names = array();
		$oldids = $products;
		$products = PAYPAL_realId($products);

        // for each item purchased, record purchase in purchase table
        for ($i = 0; $i < count($products); $i++) {
		    if ($_PAY_CONF['debug']) COM_errorLog('Product:' . $products[$i]);
            // grab relevant product data from product table to insert into purchase table.
            $sql = "SELECT * FROM {$_TABLES['paypal_products']} "
                 . "WHERE id = {$products[$i]}";
            $res = DB_query($sql);
            $A = DB_fetchArray($res);
			if ($_PAY_CONF['debug']) COM_errorLog('Type :' . $A['type']);
            if ($A['download'] > 0) {
                $files[] = $_CONF['path'] . '/plugins/paypal/files/' . $A['file'];
            }
			//TODO + attribute name
            $names[] = $product_name[$i] . ' x ' . $quantity[$i] ;

            // Do record anonymous users in purchase table
			//TODO record product name + product_id with attribute
            if ( is_numeric((int)$paypal_data['custom']) && (int)$paypal_data['custom'] > 0 ) {
                // Add the purchase to the paypal purchase table
                $sql = "INSERT INTO {$_TABLES['paypal_purchases']} SET product_id = {$products[$i]}, "
                     . "quantity = {$quantity[$i]}, user_id = {$paypal_data['custom']}, "
                     . "txn_id = '{$paypal_data['txn_id']}', "
                     . 'purchase_date = NOW(), status = \'complete\'';

                /**
                 * @todo implemente physical item vs. download, reflected in 'status'
                 */
                // if physical item (aka, must be shipped) status = 'pending', otherwise 'complete'
                //if ( $physical == 1 ) {
                //    $sql .= ", status = 'pending'";
                //} else {
                //    $sql .= ", status = 'complete'";
                //}

                // add an expiration date if appropriate
                if (is_numeric($A['expiration']) && $A['type'] == 'product') {
                    $sql .= ", expiration = DATE_ADD(NOW(), INTERVAL {$A['expiration']} DAY)";
                }
				if ($_PAY_CONF['debug']) COM_errorLog($sql);
                DB_query($sql);
				if ($_PAY_CONF['debug']) COM_errorLog('Purchase recorded');
            }
			// stock movement
			$stock_id = PAYPAL_getStockId($oldids[$i]);
			$qty = $quantity[$i];
			PAYPAL_stockMovement ($stock_id, $oldids[$i], -$qty);
        }
        
		// Send the purchaser a confirmation email (if set to do so in config)
        if ( ( is_numeric((int)$paypal_data['custom']) && (int)$paypal_data['custom'] != 1 &&
               $_PAY_CONF['purchase_email_user'] ) ||
             ( (!is_numeric($paypal_data['custom']) || (int)$paypal_data['custom'] == 1) &&
               $_PAY_CONF['purchase_email_anon'] )) {
            
			// setup templates
            $message = new Template($_CONF['path'] . 'plugins/paypal/templates');
            $message->set_file(array('subject' => 'purchase_email_subject.txt',
                                     'message' => 'purchase_email_message.txt' ));
            // site variables
            $message->set_var('site_url', $_CONF['site_url']);
            $message->set_var('site_name', $_CONF['site_name']);

			//Email subject
			$message->set_var('purchase_receipt', $LANG_PAYPAL_EMAIL['purchase_receipt']);

            // comma seperated list of product names
			for ($i = 0; $i < count($products); $i++) {
			$li_products .= '<li>' . $names[$i];
			}
			$message->set_var('products', $li_products);
			
			//Email messages
			$message->set_var('thank_you', $LANG_PAYPAL_EMAIL['thank_you']);
			$message->set_var('thanks', $LANG_PAYPAL_EMAIL['thanks']);

            // paypal details
            $message->set_var('payment_gross', $paypal_data['payment_gross']);
            $message->set_var('tax', $paypal_data['tax']);
            $message->set_var('shipping', $paypal_data['mc_shipping']);
            $message->set_var('handling', $paypal_data['mc_handling']);
            $message->set_var('payment_date', $paypal_data['payment_date']);
            $message->set_var('payer_email', $paypal_data['payer_email']);
            $message->set_var('first_name', $paypal_data['first_name']);
            $message->set_var('last_name', $paypal_data['last_name']);
			
			$subject = trim($message->parse('output', 'subject'));

            // if specified to mail attachment, do so, otherwise skip attachment
            if ( (( is_numeric((int)$paypal_data['custom']) && (int)$paypal_data['custom'] != 1 &&
                    $_PAY_CONF['purchase_email_user_attach'] ) ||
                  ( (!is_numeric((int)$paypal_data['custom']) || (int)$paypal_data['custom'] == 1) &&
                    $_PAY_CONF['purchase_email_anon_attach'] )) &&
                  count($files) > 0  ) {
				$message->set_var('attached_files', $LANG_PAYPAL_EMAIL['attached_files']);
				$text = $message->parse('output', 'message');
                paypal_mailAttachment($paypal_data['payer_email'], $subject, $text, $files,
                                      $_PAY_CONF['receiverEmailAddr']);
            } else {
			    if (count($files) > 0  ) {
			        $message->set_var('attached_files', $LANG_PAYPAL_EMAIL['download_files']);
				} else {
				    $message->set_var('attached_files', '');
				}
				$text = $message->parse('output', 'message');
                COM_mail($paypal_data['payer_email'], $subject, $text,
                         $_PAY_CONF['receiverEmailAddr'], true);
            }
			if ($_PAY_CONF['debug']) COM_errorLog('Email was sent');
        }
		//Send email to receiver
        COM_mail($_PAY_CONF['receiverEmailAddr'], $subject, $subject . ' >> ' . $text, $_PAY_CONF['receiverEmailAddr'], true);

		//Subscription
		if ($A['type'] == 'subscription') {
		    //add subscription to db
		    PAYPAL_addsubscription ($A, $paypal_data);
			if ($_PAY_CONF['debug']) COM_errorLog('Subscription recorded');
		    //add  user to group
		    ($A['add_to_group'] > 1 && (int)$paypal_data['custom'] > 1) ? PAYPAL_addToGroup($A['add_to_group'], $paypal_data['custom']) : '';
		}
    }

    /**
     * Not yet implemented
     *
     * @todo Implement handleSubscription
     */
    function handleSubscription($subscription, $paypal_data) {
		//Record subscription in DB
		
		return true;
    }

    /**
     * Not yet implemented
     *
     * @todo Implement handleDonation
     */
    function handleDonation() {
		$this->handleFailure(PAYPAL_FAILURE_UNKNOWN, "Donation not handled");
    }

    /**
     * Handle what to do in the event of a purchase/IPN failure
     *
     * This method does some basic failure handling.  For anything more
     * advanced it is recommend you override this method in IPN.class.php.
     *
     * @param int $type Type of failure that occurred
     * @param string $msg Failure message
     */
    function handleFailure($type = PAYPAL_FAILURE_UNKNOWN, $msg = '') {
        // Log the failure to geeklog's error log
        COM_errorLog('PAYPAL-IPN: ' . $msg,1);
    }
	
}
?>